﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Infrastructure;
using Microsoft.AspNetCore.Mvc.Routing;
using Microsoft.Extensions.Logging;
using GameStore.Entities.Models;
using GameStore.Services.GameStock;
using GameStore.Services.RandomId;

namespace GameStore.Api.Controllers
{
    [Route("api/[controller]")]
    //[Route("api/games")]
    public class GamesController : Controller
    {
        private readonly IGameStockService _gameStockService;
        private readonly IRandomIdService _randomId;
        private readonly IActionContextAccessor _actionContextAccessor;
        private readonly IHttpContextAccessor _httpContextAccessor;
        private readonly IUrlHelperFactory _urlHelperFactory;
        private readonly ILogger<GamesController> _logger;

        public GamesController(IGameStockService gameStockService, IRandomIdService randomId, IActionContextAccessor actionContextAccessor, IHttpContextAccessor httpContextAccessor, IUrlHelperFactory urlHelperFactory, ILogger<GamesController> logger)
        {
            _gameStockService = gameStockService;
            _randomId = randomId;
            _actionContextAccessor = actionContextAccessor;
            _httpContextAccessor = httpContextAccessor;
            _urlHelperFactory = urlHelperFactory;
            _logger = logger;
        }

        // GET api/games
        [HttpGet]
        //public async Task<IActionResult> GetAllGames(
        public async Task<IActionResult> GetAllGames(
            [FromQuery(Name = "limit")]int limit,
            [FromQuery(Name = "id_prefix")]string idPrefix,
            [FromQuery(Name = "attributes")]string[] attributes)
        {
            var games = await _gameStockService.GetAllGames();

            // Add ID prefix
            games = games.Select(game =>
            {
                game.StockItemId = idPrefix + game.StockItemId;
                return game;
            }).ToArray();

            // Apply limit
            if (limit > 0)
            {
                games = games.Take(limit).ToArray();
            }

            // Select only given fields
            if (attributes != null && attributes.Any())
            {
                games = games.Select(game =>
                {
                    var projection = new Game();

                    foreach (var attribute in attributes)
                    {
                        projection.GetType()
                            .GetProperty(attribute)
                            .SetValue(projection, game.GetType().GetProperty(attribute).GetValue(game));
                    }

                    return projection;
                }).ToArray();
            }

            _logger.LogInformation($"Retrieved {games.Length} games.");
            return Ok(games);
        }

        // GET api/games/wow
        [HttpGet("{codename}")]
        public IActionResult GetGame(string codename)
        {
            var game = _gameStockService.GetGame(codename);
            return Ok(game);
        }

        // POST api/games
        [HttpPost]
        public IActionResult Post([FromBody]Game game)
        {
            // Generate Id
            game.StockItemId = _randomId.GenerateRandomString(16);
            var result = _gameStockService.AddGame(game);

            var urlHelper = _urlHelperFactory.GetUrlHelper(_actionContextAccessor.ActionContext);
            var newUrl = new Uri(urlHelper.Action("GetGame", "Games", new
            {
                codename = game.GameCodeName,
            }, _httpContextAccessor.HttpContext.Request.Scheme));
            _logger.LogInformation("Generated new game with Id " + game.StockItemId);

            return Created(newUrl,result);
        }

        // PUT api/games/wow
        [HttpPut("{id}")]
        public void Put(int id, [FromBody]string value)
        {
        }

        // DELETE api/games/wow
        [HttpDelete("{id}")]
        public void Delete(int id)
        {
        }
    }
}
